/*!
 * Ghostery for Firefox
 * http://www.ghostery.com/
 *
 * Copyright 2013 EVIDON, Inc. All rights reserved.
 * See ghostery_eula.txt for license.
 */

var MOBILE_MODE = (require('sdk/system').platform.toLowerCase() == 'android'),
	Ci = require('chrome').Ci,
	SDK = {
		self: require('sdk/self'),
		storage: require('sdk/simple-storage').storage,
		timers: require('sdk/timers'),
		tabsLib: require('sdk/tabs/helpers'),
		tabs: require('sdk/tabs')
	},
	parseUri = require('./vendor/parseuri').parseUri;

// Android expection modules
if (MOBILE_MODE) {
	SDK.pb = null;
} else {
	SDK.pb = require('sdk/private-browsing');
}

exports.VERSION = SDK.self.version;

exports.log = function () {
	if (!exports.prefs('debug')) {
		return;
	}
	// convert arguments object to array
	var args = [].slice.call(arguments, 0);
	if (args[0]) {
		args[0] = (new Date()).toTimeString() + '\t\t' + args[0];
	}
	console.log.apply(console, args);
};

exports.prefs = function (key, value) {
	if (typeof value != 'undefined') {
		SDK.storage[key] = value;
	} else {
		return SDK.storage[key];
	}
};

// TODO replace with _.memoize
exports.syncGet = (function () {
	var memo = {};

	return function (url, memoize) {
		var result;

		if (memoize && memo.hasOwnProperty(url)) {
			return memo[url];
		}

		result = SDK.self.data.load(url);
		if (memoize) {
			memo[url] = result;
		}

		return result;
	};
}());

exports.fuzzyUrlMatcher = function (url, urls) {
	// TODO belongs in tabInfo
	var parsed = parseUri(url),
		tab_host = parsed.host,
		tab_path = parsed.path;

	if (tab_host.indexOf('www.') === 0) {
		tab_host = tab_host.slice(4);
	}

	for (var i = 0; i < urls.length; i++) {
		parsed = parseUri(urls[i]);
		var host = parsed.host,
			path = parsed.path;

		if (host != tab_host) {
			continue;
		}

		if (!path) {
			exports.log('[fuzzyUrlMatcher] host (%s) match', host);
			return true;
		}

		if (path.slice(-1) == '*') {
			if (tab_path.indexOf(path.slice(0, -1)) === 0) {
				exports.log('[fuzzyUrlMatcher] host (%s) and path (%s) fuzzy match', host, path);
				return true;
			}

		} else {
			if (path == tab_path) {
				exports.log('[fuzzyUrlMatcher] host (%s) and path (%s) match', host, path);
				return true;
			}
		}
	}
};

exports.blockingLog = function blockingLog(msg, type, context, blockLogWorker) {
	if (!blockLogWorker || (SDK.pb && SDK.pb.isPrivate(context))) { return; }

	SDK.timers.setTimeout(function () {
		switch (type) {
		case 2:
			type = 'script';
			break;
		case 3:
			type = 'image';
			break;
		case 5:
			type = 'object';
			break;
		case 7:
			type = 'frame';
			break;
		case 12:
			type = 'sub-object';
			break;
		default:
			type = 'unknown-type';
			break;
		}
		
		msg = msg.replace('$TYPE$', type);
		try {
			blockLogWorker.port.emit('blockingLogUpdate', {
				blockingLog: msg
			});
		} catch (e) {}
	}, 1);
};

exports.getTabForContext = function (context) {
	// If it is the main frame
	if (context._contentWindow instanceof Ci.nsIDOMWindow) {
		return SDK.tabsLib.getTabForWindow(context._contentWindow.top);
	}

	if (!(context instanceof Ci.nsIDOMWindow)) {
		// If this is an element, get the corresponding document
		if (context instanceof Ci.nsIDOMNode && context.ownerDocument) {
			context = context.ownerDocument;
		}
		// Now we should have a document, get its window
		if (context instanceof Ci.nsIDOMDocument) {
			context = context.defaultView;
		} else {
			context = null;
		}
	}

	// If we have a window now - get the tab
	if (context) {
		return SDK.tabsLib.getTabForWindow(context.top);
	} else {
		return null;
	}
};

exports.getTabFromChannel = function (aChannel) {
	try {
		var notificationCallbacks = aChannel.notificationCallbacks ? aChannel.notificationCallbacks : aChannel.loadGroup.notificationCallbacks;
		if (!notificationCallbacks) {
			return null;
		}

		return SDK.tabsLib.getTabForWindow(notificationCallbacks.getInterface(Ci.nsIDOMWindow));
	} catch (e) {
		return null;
	}
};

exports.reloadTab = function (tab_id) {
	var tab = getTab(tab_id, function (tab) {
		tab.reload();
		return true;
	});

	if (!tab && SDK.tabs.activeTab) {
		SDK.tabs.activeTab.reload();
	}
};

function getTab(tab_id, callback) {
	var tab, i;

	for (i = 0; i < SDK.tabs.length; i++) {
		tab = SDK.tabs[i];
		if (tab.id == tab_id) {
			return (callback) ? callback(tab) : tab;
		}
	}

	return false;
}
exports.getTab = getTab;